// namespace, needed for Firefox
if ("undefined" == typeof(BFolders)) {
  var BFolders = {};
};

BFolders._debug = false;

BFolders.log = function (message) {
    if (!this._debug)
        return;
    // dump("B-Folders: " + message + "\n");
    //Services.console.logStringMessage("B-Folders: " + message);
}

BFolders.INCOMPATIBILITY_NOTIFICATION_ID = 'b-folders-incompatibility';
BFolders.RETRY_NOTIFICATION_ID = 'b-folders-retry';
BFolders.NOTIFICATION_ICON = "skin/b-folders-128.png";

BFolders._callbackToRetry = null;
BFolders._isLinux = false;

disableChromePasswordManager();

// get messages from the content script
chrome.runtime.onMessage.addListener(
		  function(request, sender, sendResponse) {
			  var cmd = request[0];
			  var param = request[1];
			  if("loginCmd" == cmd) {
				  BFolders.sendQueryCommand(param);
			  } else if ("generatePasswordCmd" == cmd) {
				  BFolders.sendGeneratePasswordCommand(param);
			  } else if ("handleLoginCapturedCmd" == cmd) {
				  BFolders.handleLoginCaptured(param);
			  }
		  });

chrome.notifications.onButtonClicked.addListener(
		function(notificationId, buttonIndex) {
			// we use only one button, hence we do not check buttonIndex
			BFolders.handleNotificationAccepted(notificationId);
});

// !! WORKAROUND - On Linux buttons are not displayed, because of a bug in Chrome, so we 
// hook to clicking on the notification itself
// TODO - remove when the bug is fixed
chrome.runtime.getPlatformInfo(function(platformInfo){
	if (platformInfo.os == "linux") {
		BFolders._isLinux = true;
		
		chrome.notifications.onClicked.addListener(
			function(notificationId) {
				BFolders.handleNotificationAccepted(notificationId);				
			});
	}
});


BFolders.handleNotificationAccepted = function(notificationId) {
	// auto-close when a button is clicked - like Mozilla Firefox
	chrome.notifications.clear(notificationId, function(wasCleared) {});
	
	if (this.RETRY_NOTIFICATION_ID == notificationId) {
		if (this._callbackToRetry == null)
			return;
		this._callbackToRetry.call();
	}
}

// called during scope initialization above
function disableChromePasswordManager() {
	// TODO this does not work! - it disables the "	Enable Autofill to fill out web forms in a single click."
	// setting by it does not prevent Chrome from autofilling passwords. Only seems to affect autofilling addresses and CCs only.
	
	// !!! If trying this set the "privacy" permission in the manifest.
/*	
	window.alert("checkpoint");
	chrome.privacy.services.autofillEnabled.get({}, function(details) {
		window.alert("level of control:" + details.levelOfControl);
		
		if (details.levelOfControl == 'controllable_by_this_extension'
			|| details.levelOfControl == 'controlled_by_this_extension') {
			window.alert("Can controll privacy");
			if (details.value) {
				window.alert("Autofill is set");
				chrome.privacy.services.autofillEnabled.set({ value: false }, function() {
				      if (chrome.runtime.lastError === undefined) {
				    	  window.alert("OK");
				      } else {
				    	  window.alert("Failed");
				      }
				});
			}
		}
	});
	*/
}

BFolders.doContentAutofillLogin = function(login) {
	chrome.tabs.query({active: true, currentWindow: true}, function(tabs) {
		var msg = ["autofillLogin", login]
		chrome.tabs.sendMessage(tabs[0].id, msg, function(response) {
		  });
	});
}

BFolders.doContentAutofillGeneratedPassword = function(password) {
	chrome.tabs.query({active: true, currentWindow: true}, function(tabs) {
		var msg = ["autofillGeneratedPassword", password]
		chrome.tabs.sendMessage(tabs[0].id, msg, function(response) {
		  });
	});
}

BFolders.createXMLHttpRequest = function() {
	return new XMLHttpRequest();
}

BFolders.askToRetry = function(message, pathAndQueryString, responseCallback) {
	// Explicitly close the notification to make sure the pop-up window is displayed if
	// the same notification is already displayed. 
	// If chrome.notifications.create() replaces existing notification it does not show the pop-up window.
	this.closeAskToRetry();
	
	this._callbackToRetry = function() {
		BFolders.sendCommand(pathAndQueryString, responseCallback, true);
	}

	var opt;
	if (brand == "firefox") {
		// Firefox does not suppot buttons property
		opt = {
				  type: "basic",
				  title: "",
				  message: message,
				  iconUrl: this.NOTIFICATION_ICON
				}
	} else {
		opt = {
				  type: "basic",
				  title: "",
				  buttons: [{title: "Retry"}],
				  message: message,
				  iconUrl: this.NOTIFICATION_ICON
				}	
	}
	
	
	
	// TODO remove when the bug is fixed
	if (this._isLinux) {
		opt.isClickable = true; 
	}
	
	chrome.notifications.create(this.RETRY_NOTIFICATION_ID, opt, function(notificationId) {});
	
}

BFolders.closeAskToRetry = function() {
	chrome.notifications.clear(this.RETRY_NOTIFICATION_ID, function(wasCleared) {});
}
